local L = LibStub("AceLocale-3.0"):GetLocale("ShockAndAwe")
local media = LibStub:GetLibrary("LibSharedMedia-3.0");
local AceEvent = LibStub("AceEvent-3.0")

-- Binding Variables
BINDING_HEADER_SHOCKANDAWE_TITLE = L["Keybind Title"]
BINDING_NAME_SHOCKANDAWE_WATER_SHIELD = L["WaterShield"]
BINDING_NAME_SHOCKANDAWE_LIGHTNING_SHIELD = L["LightningShield"]
BINDING_NAME_SHOCKANDAWE_WEAPON_REBUFF = L["Weapon Rebuff"]

ShockAndAwe.BindingFrame = CreateFrame("Frame", "SAA_BindingFrame", UIParent)
ShockAndAwe.bindings = {}
ShockAndAwe.Button = {}
--------------------------------
--  Create Binding Frame
--------------------------------

function ShockAndAwe:CreateBindingFrame()
	self.BindingFrame:SetScale(self.db.char.binding.scale)
	self.BindingFrame:SetFrameStrata("BACKGROUND")
	self.BindingFrame:SetWidth(self.db.char.binding.fWidth)
	self.BindingFrame:SetHeight(self.db.char.binding.fHeight)
	self.BindingFrame:SetBackdrop({ bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 0, edgeSize = 12,
		insets = { left = 2, right = 2, top = 2, bottom = 2 }
	})
	self.BindingFrame:SetBackdropColor(0, 0, 0, self.db.char.binding.alpha);
	self.BindingFrame:SetMovable(true);
	self.BindingFrame:RegisterForDrag("LeftButton");
	self.BindingFrame:SetPoint(self.db.char.binding.point, self.db.char.binding.relativeTo, self.db.char.binding.relativePoint, self.db.char.binding.xOffset, self.db.char.binding.yOffset)
	self.BindingFrame:SetScript("OnDragStart", 
		function()
			self.BindingFrame:StartMoving();
		end );
	self.BindingFrame:SetScript("OnDragStop",
		function()
			self.BindingFrame:StopMovingOrSizing();
			self.BindingFrame:SetScript("OnUpdate", nil);
			self:FinishedMoving(self.db.char.binding, self.BindingFrame);
		end );
		
	if not self.BindingFrame.topText then
		self.BindingFrame.topText = self.BindingFrame:CreateFontString(nil, "OVERLAY")
	end
	self.BindingFrame.topText:SetTextColor(1,1,1,1)
	self.BindingFrame.topText:SetFont(media:Fetch("font", self.db.char.barfont), self.db.char.barfontsize)
	self.BindingFrame.topText:SetPoint("TOP", self.BindingFrame, "TOP", 0, self.db.char.barfontsize + 2)
	self.BindingFrame.topText:SetText("keybound buttons")

	if self.db.char.binding.show then
		self.BindingFrame:Show()
	else
		self.BindingFrame:Hide()
	end
end

--------------------------------
--  Create Button Template
--------------------------------

function ShockAndAwe.Button:CreateBase(name, parent)
	local button = CreateFrame("Button", name, parent, "SecureActionButtonTemplate, ActionButtonTemplate") --SecureHandlerEnterLeaveTemplate, SecureHandlerAttributeTemplate,
	button.Parent = parent
	button.Name = name
	button:SetWidth(36)
	button:SetHeight(36)	
	button:RegisterForClicks("LeftButtonUp","RightButtonUp")		
	
--	button:HookScript("OnEnter", function() button:OnEnter(button) end)
--	button:HookScript("OnLeave", function() button:OnLeave() end)
	
	button.Icon = button.Icon or button:CreateTexture(name.."Icon")   
	button.Icon:SetAllPoints(button)
	
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
	button:GetHighlightTexture():SetBlendMode("ADD")
	
	button.Cooldown = button.Cooldown or CreateFrame("Cooldown", name.."Cooldown",button,"CooldownFrameTemplate")
	button.Cooldown:SetWidth(36)
	button.Cooldown:SetHeight(36)
	button.Cooldown:SetPoint("CENTER",0,-1)
	
	button.HotKey = button.HotKey or button:CreateFontString(name.."HotKey", "OVERLAY", "NumberFontNormalSmallGray")
	button.HotKey:SetWidth(36)
	button.HotKey:SetHeight(10)
	button.HotKey:SetJustifyH("RIGHT")
	button.HotKey:SetPoint("TOPLEFT",-2,-2)	
	
	button:SetNormalTexture("")	
	button.NormalTexture = button:GetNormalTexture()
	button.NormalTexture:SetWidth(66)
	button.NormalTexture:SetHeight(66)
	button.NormalTexture:ClearAllPoints()
	button.NormalTexture:SetPoint("CENTER", 0, -1)
	button.NormalTexture:Hide()

	button.Duration = button.Duration or button:CreateFontString(name.."Duration", "BACKGROUND", "GameFontNormalSmall")
	button.Duration:SetPoint("TOP",parent,"BOTTOM",0,0)	
	
--	button:RegisterEvents()
--	button:UpdateHotkey()	
	return button
end

function ShockAndAwe.Button:CreateButton(name, parent)
	local button = ShockAndAwe.Button:CreateBase(name,parent)
	button:SetParent(parent)
	button:SetAttribute("hidden", false)
	return button		
end

function ShockAndAwe:InitialiseBindings()
	self:CreateBindingFrame()
	self.BindingFrame:SetFrameStrata("HIGH")
	self.BindingFrame:SetWidth(250)
	self.BindingFrame:SetHeight(50)
	self.BindingFrame:ClearAllPoints()
	self.BindingFrame:SetPoint("TOPLEFT", self.BaseFrame, "TOPLEFT", barHeight, frameOffset)
	self.BindingFrame:SetBackdrop(self.barBackdrop)
	self.BindingFrame:SetBackdropColor(0, 0, 0, .2)
	self.BindingFrame:SetBackdropBorderColor( 1, 1, 1, 1)
	self.BindingFrame:EnableMouse(true)
	self.bindings.watershield = self.Button:CreateButton("SAA_WaterShieldButton", self.BindingFrame)
	self.bindings.lightningshield = self.Button:CreateButton("SAA_LightningShieldButton", self.BindingFrame)
	self.bindings.weaponrebuff = self.Button:CreateButton("SAA_WeaponRebuffButton", self.BindingFrame)
	
	self:UpdateBindings()
	self.BindingFrame:Show()
end

function ShockAndAwe:UpdateBindings()
	ClearOverrideBindings(self.bindings.watershield)
	local key = GetBindingKey("SHOCKANDAWE_WATER_SHIELD")
	if (key) then
		SetOverrideBindingClick(self.bindings.watershield, nil, key, "SAA_WaterShieldButton")
	end
	self:UpdateButton(self.bindings.watershield, L["WaterShield"])
	
	ClearOverrideBindings(self.bindings.lightningshield)
	local key = GetBindingKey("SHOCKANDAWE_LIGHTNING_SHIELD")
	if (key) then
		SetOverrideBindingClick(self.bindings.lightningshield, nil, key, "SAA_LightningShieldButton")
	end
	self:UpdateButton(self.bindings.lightningshield, L["LightningShield"])

	ClearOverrideBindings(self.bindings.weaponrebuff)
	local key = GetBindingKey("SHOCKANDAWE_WEAPON_REBUFF")
	if (key) then
		SetOverrideBindingClick(self.bindings.weaponrebuff, nil, key, "SAA_WeaponRebuffButton")
	end
	self:UpdateButton(self.bindings.weaponrebuff, self.db.char.binding.mhspell)
	
	self.BindingFrame:Hide()
end

function ShockAndAwe:UpdateButton(button, spell)
	button:SetAttribute("type1", "spell")
	button:SetAttribute("spell1", spell)
	local _, _, texture = GetSpellInfo(spell)
	button.Icon:SetTexture(texture)
	button:Enable()
	button:Show()
	button:SetNormalTexture(nil);
end

-----------------------------
-- Button press functions
-----------------------------

function ShockAndAwe:CastWaterShield()
	self.bindings.watershield:Click()
end

function ShockAndAwe:CastLightningShield()
	self.bindings.lightningshield:Click()
end

function ShockAndAwe:RebuffWeapons()
	self:Print("entered rebuff Weapons")
	self:Print("mh :"..self.db.char.binding.mhspell.." oh:"..self.db.char.binding.ohspell)
	if self:MainHandBuffMissing() then
		self:Print("updating main hand")
		self:UpdateButton(self.bindings.weaponrebuff, self.db.char.binding.mhspell)
		self.bindings.weaponrebuff:Click()
	end
	if self:OffHandBuffMissing() then
		self:Print("updating off hand")
		self:UpdateButton(self.bindings.weaponrebuff, self.db.char.binding.ohspell)
		self.bindings.weaponrebuff:Click()
	end
end

function ShockAndAwe:MainHandBuffMissing()
	return true
end

function ShockAndAwe:OffHandBuffMissing()
	return true
end